@@ -1,14 +1,18 @@
 #!/usr/bin/python3
+
+#   1. Standard library imports.
 import gettext
-import gi
 import locale
 import os
-import re
-import setproctitle
 import shutil
 import subprocess
-import tldextract
 import warnings
+import webbrowser
+
+#   2. Related third party imports.
+import gi
+import setproctitle
+import tldextract
 
 # Suppress GTK deprecation warnings
 warnings.filterwarnings("ignore")
@@ -17,6 +21,7 @@ gi.require_version("Gtk", "3.0")
 gi.require_version('XApp', '1.0')
 from gi.repository import Gtk, Gdk, Gio, XApp, GdkPixbuf, GLib
 
+#   3. Local application/library specific imports.
 from common import _async, idle, WebAppManager, Browser, download_favicon, ICONS_DIR, BROWSER_TYPE_FIREFOX
 
 setproctitle.setproctitle("webapp-manager")
@@ -33,6 +38,7 @@ COL_ICON, COL_NAME, COL_BROWSER, COL_WEBAPP = range(4)
 CATEGORY_ID, CATEGORY_NAME = range(2)
 BROWSER_OBJ, BROWSER_NAME = range(2)
 
+
 class MyApplication(Gtk.Application):
     # Main initialization routine
     def __init__(self, application_id, flags):
@@ -41,7 +47,7 @@ class MyApplication(Gtk.Application):
 
     def activate(self, application):
         windows = self.get_windows()
-        if (len(windows) > 0):
+        if len(windows) > 0:
             window = windows[0]
             window.present()
             window.show()
@@ -50,7 +56,8 @@ class MyApplication(Gtk.Application):
             self.add_window(window.window)
             window.window.show()
 
-class WebAppManagerWindow():
+
+class WebAppManagerWindow:
 
     def __init__(self, application):
 
@@ -85,6 +92,7 @@ class WebAppManagerWindow():
         self.name_entry = self.builder.get_object("name_entry")
         self.url_entry = self.builder.get_object("url_entry")
         self.url_label = self.builder.get_object("url_label")
+        self.customparameters_entry = self.builder.get_object("customparameters_entry")
         self.isolated_switch = self.builder.get_object("isolated_switch")
         self.isolated_label = self.builder.get_object("isolated_label")
         self.navbar_switch = self.builder.get_object("navbar_switch")
@@ -97,10 +105,7 @@ class WebAppManagerWindow():
         self.browser_label = self.builder.get_object("browser_label")
 
         # Widgets which are in the add page but not the edit page
-        self.add_specific_widgets = [self.browser_label, self.browser_combo,
-                                     self.isolated_label, self.isolated_switch,
-                                     self.navbar_label, self.navbar_switch,
-                                     self.privatewindow_label, self.privatewindow_switch]
+        self.add_specific_widgets = [self.browser_label, self.browser_combo]
 
         # Widget signals
         self.add_button.connect("clicked", self.on_add_button)
@@ -113,14 +118,15 @@ class WebAppManagerWindow():
         self.favicon_button.connect("clicked", self.on_favicon_button)
         self.name_entry.connect("changed", self.on_name_entry)
         self.url_entry.connect("changed", self.on_url_entry)
-        self.window.connect("key-press-event",self.on_key_press_event)
+        self.window.connect("key-press-event", self.on_key_press_event)
 
         # Menubar
         accel_group = Gtk.AccelGroup()
         self.window.add_accel_group(accel_group)
         menu = self.builder.get_object("main_menu")
         item = Gtk.ImageMenuItem()
-        item.set_image(Gtk.Image.new_from_icon_name("preferences-desktop-keyboard-shortcuts-symbolic", Gtk.IconSize.MENU))
+        item.set_image(
+            Gtk.Image.new_from_icon_name("preferences-desktop-keyboard-shortcuts-symbolic", Gtk.IconSize.MENU))
         item.set_label(_("Keyboard Shortcuts"))
         item.connect("activate", self.open_keyboard_shortcuts)
         key, mod = Gtk.accelerator_parse("<Control>K")
@@ -163,31 +169,31 @@ class WebAppManagerWindow():
         self.treeview.append_column(column)
 
         self.treeview.show()
-        self.model = Gtk.TreeStore(GdkPixbuf.Pixbuf, str, str, object) # icon, name, browser, webapp
+        self.model = Gtk.TreeStore(GdkPixbuf.Pixbuf, str, str, object)  # icon, name, browser, webapp
         self.model.set_sort_column_id(COL_NAME, Gtk.SortType.ASCENDING)
         self.treeview.set_model(self.model)
         self.treeview.get_selection().connect("changed", self.on_webapp_selected)
         self.treeview.connect("row-activated", self.on_webapp_activated)
 
         # Combox box
-        category_model = Gtk.ListStore(str,str) # CATEGORY_ID, CATEGORY_NAME
-        category_model.append(["Network",_("Internet")])
-        category_model.append(["WebApps",_("Web")])
-        category_model.append(["Utility",_("Accessories")])
-        category_model.append(["Game",_("Games")])
-        category_model.append(["Graphics",_("Graphics")])
-        category_model.append(["Office",_("Office")])
-        category_model.append(["AudioVideo",_("Sound & Video")])
-        category_model.append(["Development",_("Programming")])
-        category_model.append(["Education",_("Education")])
+        category_model = Gtk.ListStore(str, str)  # CATEGORY_ID, CATEGORY_NAME
+        category_model.append(["WebApps", _("Web")])
+        category_model.append(["Network", _("Internet")])
+        category_model.append(["Utility", _("Accessories")])
+        category_model.append(["Game", _("Games")])
+        category_model.append(["Graphics", _("Graphics")])
+        category_model.append(["Office", _("Office")])
+        category_model.append(["AudioVideo", _("Sound & Video")])
+        category_model.append(["Development", _("Programming")])
+        category_model.append(["Education", _("Education")])
         self.category_combo = self.builder.get_object("category_combo")
         renderer = Gtk.CellRendererText()
         self.category_combo.pack_start(renderer, True)
         self.category_combo.add_attribute(renderer, "text", CATEGORY_NAME)
         self.category_combo.set_model(category_model)
-        self.category_combo.set_active(0) # Select 1st category
+        self.category_combo.set_active(0)  # Select 1st category
 
-        browser_model = Gtk.ListStore(object, str) # BROWSER_OBJ, BROWSER_NAME
+        browser_model = Gtk.ListStore(object, str)  # BROWSER_OBJ, BROWSER_NAME
         num_browsers = 0
         for browser in self.manager.get_supported_browsers():
             if os.path.exists(browser.test_path):
@@ -197,12 +203,12 @@ class WebAppManagerWindow():
         self.browser_combo.pack_start(renderer, True)
         self.browser_combo.add_attribute(renderer, "text", BROWSER_NAME)
         self.browser_combo.set_model(browser_model)
-        self.browser_combo.set_active(0) # Select 1st browser
+        self.browser_combo.set_active(0)  # Select 1st browser
         if num_browsers == 0:
-        	print ("No supported browsers were detected.")
-        	self.add_button.set_sensitive(False)
-        	self.add_button.set_tooltip_text(_("No supported browsers were detected."))
-        if (num_browsers < 2):
+            print("No supported browsers were detected.")
+            self.add_button.set_sensitive(False)
+            self.add_button.set_tooltip_text(_("No supported browsers were detected."))
+        if num_browsers < 2:
             self.browser_label.hide()
             self.browser_combo.hide()
         self.browser_combo.connect("changed", self.on_browser_changed)
@@ -241,15 +247,17 @@ class WebAppManagerWindow():
             h.close()
             dlg.set_license(gpl)
         except Exception as e:
-            print (e)
+            print(e)
 
         dlg.set_version("__DEB_VERSION__")
         dlg.set_icon_name("webapp-manager")
         dlg.set_logo_icon_name("webapp-manager")
         dlg.set_website("https://www.github.com/linuxmint/webapp-manager")
+
         def close(w, res):
             if res == Gtk.ResponseType.CANCEL or res == Gtk.ResponseType.DELETE_EVENT:
                 w.destroy()
+
         dlg.connect("response", close)
         dlg.show()
 
@@ -280,7 +288,7 @@ class WebAppManagerWindow():
             self.load_webapps()
 
     def on_remove_button(self, widget):
-        if self.selected_webapp != None:
+        if self.selected_webapp is not None:
             dialog = Gtk.MessageDialog(message_type=Gtk.MessageType.WARNING)
             dialog.set_transient_for(self.window)
             dialog.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_DELETE, Gtk.ResponseType.YES)
@@ -294,7 +302,7 @@ class WebAppManagerWindow():
             dialog.destroy()
 
     def run_webapp(self, webapp):
-        if webapp != None:
+        if webapp is not None:
             print("Running %s" % webapp.path)
             print("Executing %s" % webapp.exec)
             subprocess.Popen(webapp.exec, shell=True)
@@ -311,6 +319,7 @@ class WebAppManagerWindow():
         navbar = self.navbar_switch.get_active()
         privatewindow = self.privatewindow_switch.get_active()
         icon = self.icon_chooser.get_icon()
+        custom_parameters = self.customparameters_entry.get_text()
         if "/tmp" in icon:
             # If the icon path is in /tmp, move it.
             filename = "".join(filter(str.isalpha, name)) + ".png"
@@ -318,15 +327,17 @@ class WebAppManagerWindow():
             shutil.copyfile(icon, new_path)
             icon = new_path
         if self.edit_mode:
-            self.manager.edit_webapp(self.selected_webapp.path, name, browser, url, icon, category)
+            self.manager.edit_webapp(self.selected_webapp.path, name, browser, url, icon, category, custom_parameters, self.selected_webapp.codename, isolate_profile, navbar, privatewindow)
             self.load_webapps()
         else:
-            self.manager.create_webapp(name, url, icon, category, browser, isolate_profile, navbar, privatewindow)
+            self.manager.create_webapp(name, url, icon, category, browser, custom_parameters, isolate_profile, navbar,
+                                       privatewindow)
             self.load_webapps()
 
     def on_add_button(self, widget):
         self.name_entry.set_text("")
         self.url_entry.set_text("")
+        self.customparameters_entry.set_text("")
         self.icon_chooser.set_icon("webapp-manager")
         self.category_combo.set_active(0)
         self.browser_combo.set_active(0)
@@ -343,10 +354,20 @@ class WebAppManagerWindow():
         self.name_entry.grab_focus()
 
     def on_edit_button(self, widget):
-        if self.selected_webapp != None:
+        if self.selected_webapp is not None:
             self.name_entry.set_text(self.selected_webapp.name)
             self.icon_chooser.set_icon(self.selected_webapp.icon)
             self.url_entry.set_text(self.selected_webapp.url)
+            self.customparameters_entry.set_text(self.selected_webapp.custom_parameters)
+            self.navbar_switch.set_active(self.selected_webapp.navbar)
+            self.isolated_switch.set_active(self.selected_webapp.isolate_profile)
+            self.privatewindow_switch.set_active(self.selected_webapp.privatewindow)
+
+            web_browsers = map(lambda i: i[0], self.browser_combo.get_model())
+            selected_browser_index = [idx for idx, x in enumerate(web_browsers) if x.name == self.selected_webapp.web_browser][0]
+            self.browser_combo.set_active(selected_browser_index)
+            self.on_browser_changed(self.selected_webapp)
+
             model = self.category_combo.get_model()
             iter = model.get_iter_first()
             while iter:
@@ -355,6 +376,7 @@ class WebAppManagerWindow():
                     self.category_combo.set_active_iter(iter)
                     break
                 iter = model.iter_next(iter)
+            self.show_hide_browser_widgets()
             for widget in self.add_specific_widgets:
                 widget.hide()
             self.stack.set_visible_child_name("add_page")
@@ -431,7 +453,7 @@ class WebAppManagerWindow():
 
     def show_hide_browser_widgets(self):
         browser = self.browser_combo.get_model()[self.browser_combo.get_active()][BROWSER_OBJ]
-        if (browser.browser_type == BROWSER_TYPE_FIREFOX):
+        if browser.browser_type == BROWSER_TYPE_FIREFOX:
             self.isolated_label.hide()
             self.isolated_switch.hide()
             self.navbar_label.show()
@@ -468,9 +490,9 @@ class WebAppManagerWindow():
         if url != "":
             info = tldextract.extract(url)
             icon = None
-            if info.domain == None or info.domain == "":
+            if info.domain is None or info.domain == "":
                 return
-            if info.domain == "google" and info.subdomain != None and info.subdomain != "":
+            if info.domain == "google" and info.subdomain is not None and info.subdomain != "":
                 if info.subdomain == "mail":
                     icon = "web-%s-gmail" % info.domain
                 else:
@@ -479,7 +501,7 @@ class WebAppManagerWindow():
                 icon = "web-google-gmail"
             elif info.domain == "youtube":
                 icon = "web-google-youtube"
-            if icon != None and self.icon_theme.has_icon(icon):
+            if icon is not None and self.icon_theme.has_icon(icon):
                 self.icon_chooser.set_icon(icon)
             elif self.icon_theme.has_icon("web-%s" % info.domain):
                 self.icon_chooser.set_icon("web-%s" % info.domain)
@@ -498,7 +520,8 @@ class WebAppManagerWindow():
         for webapp in webapps:
             if webapp.is_valid:
                 if "/" in webapp.icon and os.path.exists(webapp.icon):
-                    pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(webapp.icon, -1, 32 * self.window.get_scale_factor())
+                    pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(webapp.icon, -1,
+                                                                    32 * self.window.get_scale_factor())
                 else:
                     if self.icon_theme.has_icon(webapp.icon):
                         pixbuf = self.icon_theme.load_icon(webapp.icon, 32 * self.window.get_scale_factor(), 0)
@@ -523,3 +546,4 @@ class WebAppManagerWindow():
 if __name__ == "__main__":
     application = MyApplication("org.x.webapp-manager", Gio.ApplicationFlags.FLAGS_NONE)
     application.run()
+
